Package: Coralysis
Type: Package
Title: Coralysis sensitive identification of imbalanced cell types and
        states in single-cell data via multi-level integration
Version: 1.3.0
Description: Coralysis is an R package featuring a multi-level integration algorithm for sensitive integration, 
            reference-mapping, and cell-state identification in single-cell data. The multi-level integration 
            algorithm is inspired by the process of assembling a puzzle - where one begins by grouping pieces 
            based on low-to high-level features, such as color and shading, before looking into shape and patterns. 
            This approach progressively blends the batch effects and separates cell types across multiple rounds 
            of divisive clustering.
Authors@R: c(person("António", "Sousa",
             email = "aggode@utu.fi",
             role=c("cre", "aut"), 
             comment = c(ORCID = "0000-0003-4779-6459")),
             person("Johannes", "Smolander",
             role=c("ctb", "aut"), 
             comment = c(ORCID = "0000-0003-3872-9668")),
             person("Sini", "Junttila",
             role=c("aut"), 
             comment = c(ORCID = "0000-0003-3754-5584")),
             person("Laura L", "Elo",
             role=c("aut"), 
             comment = c(ORCID = "0000-0001-5648-4532")))
License: GPL-3
Imports: Matrix, aricode, LiblineaR, SparseM, ggplot2, umap, Rtsne,
        pheatmap, reshape2, dplyr, SingleCellExperiment,
        SummarizedExperiment, S4Vectors, methods, stats, utils, RANN,
        sparseMatrixStats, irlba, flexclust, scran, class, matrixStats,
        tidyr, cowplot, uwot, scatterpie, RColorBrewer, ggrastr,
        ggrepel, RSpectra, BiocParallel, withr
Depends: R (>= 4.2.0)
Suggests: knitr, rmarkdown, bluster, ComplexHeatmap, circlize, scater,
        viridis, scRNAseq, SingleR, MouseGastrulationData, testthat (>=
        3.0.0), BiocStyle, scrapper
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
biocViews: SingleCell, RNASeq, Proteomics, Transcriptomics,
        GeneExpression, BatchEffect, Clustering, Annotation,
        Classification, DifferentialExpression, DimensionReduction,
        Software
NeedsCompilation: no
URL: https://github.com/elolab/Coralysis,
        https://elolab.github.io/Coralysis/
BugReports: https://github.com/elolab/Coralysis/issues
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/Coralysis
git_branch: devel
git_last_commit: de2d3ab
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 22:09:49 UTC; biocbuild
Author: António Sousa [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-4779-6459>),
  Johannes Smolander [ctb, aut] (ORCID:
    <https://orcid.org/0000-0003-3872-9668>),
  Sini Junttila [aut] (ORCID: <https://orcid.org/0000-0003-3754-5584>),
  Laura L Elo [aut] (ORCID: <https://orcid.org/0000-0001-5648-4532>)
Maintainer: António Sousa <aggode@utu.fi>
