# Generated by roxygen2: do not edit by hand

export(AggregateDataByBatch)
export(BinCellClusterProbability)
export(CellBinsFeatureCorrelation)
export(CellClusterProbabilityDistribution)
export(FindAllClusterMarkers)
export(FindClusterMarkers)
export(GetCellClusterProbability)
export(GetFeatureCoefficients)
export(HeatmapFeatures)
export(MajorityVotingFeatures)
export(PCAElbowPlot)
export(PlotClusterTree)
export(PlotDimRed)
export(PlotExpression)
export(PrepareData)
export(ReferenceMapping)
export(RunPCA)
export(RunParallelDivisiveICP)
export(RunTSNE)
export(RunUMAP)
export(SummariseCellClusterProbability)
export(TabulateCellBinsByGroup)
export(VlnPlot)
import(SparseM)
import(ggplot2)
import(pheatmap)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpstop)
importFrom(BiocParallel,bpworkers)
importFrom(LiblineaR,LiblineaR)
importFrom(Matrix,Matrix)
importFrom(RANN,nn2)
importFrom(RSpectra,eigs_sym)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(SingleCellExperiment,"logcounts<-")
importFrom(SingleCellExperiment,"reducedDim<-")
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,cbind)
importFrom(SingleCellExperiment,logcounts)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SummarizedExperiment,"assayNames<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(aricode,clustComp)
importFrom(cowplot,plot_grid)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_at)
importFrom(dplyr,n)
importFrom(dplyr,ntile)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(flexclust,kcca)
importFrom(flexclust,kccaFamily)
importFrom(irlba,prcomp_irlba)
importFrom(methods,as)
importFrom(methods,is)
importFrom(reshape2,melt)
importFrom(scran,getTopHVGs)
importFrom(sparseMatrixStats,colMeans2)
importFrom(sparseMatrixStats,colSds)
importFrom(sparseMatrixStats,rowMeans2)
importFrom(sparseMatrixStats,rowSds)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,wilcox.test)
importFrom(withr,with_seed)
